/**
 * The `ep.productUiTaxExceptions` widget creates widget which lets the user
 * choose exceptions from the tax country list.
 *
 * ### Examples
 * Creates a dialog for tax exceptions.
 *
 * JavaScript:
 *
 *    $('.ep-tax-exceptions-link').productUiTaxExceptions({
 *        objectId: #ID,
 *        changeAction: 'JSONSaveTaxClassMapping',
 *        taxClassCountries: #IF(#DEFINED(#TaxClassCountries))#TO_JSON(#TaxClassCountries)#ELSE#SPACE[0]{preselected: [], optional: []}#ENDIF,
 *        taxClassMappings: #IF(#DEFINED(#TaxClassMappings))#TO_JSON(#TaxClassMappings)#ELSE#SPACE[0][]#ENDIF,
 *        selectors: {
 *            taxClassSelect: '#' + 'TaxClass'
 *        },
 *        autoOpen: false,
 *        debug: true
 *    });
 *
 * HTML:
 *
 *     <a class="ep-tax-exceptions-link">Exceptions</a>
 *
 *
 * @class jQuery.ui.productUiTaxExceptions
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses jquery.ui.widget
 * @uses jquery.tmpl
 * @uses jquery.i18n
 * @uses ep.alert
 * @uses ep.ui.dialog
 *
 * @since 7.63.0
 */

/**
 * Method to create and instantiate the dialog widget.
 *
 * @method _createDialog
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to send data via an ajax request to the server.
 *
 * @method _send
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.64.0
 */

/**
 * Method to change tax area and its corresponding tax classes. And trigger a re-rendering.
 *
 * @method _handleTaxAreaSelection
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to change tax class for chosen tax area (country).
 *
 * @method _handleTaxClassSelection
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to handle add button.
 *
 * @method _handleAddButton
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to handle delete button.
 *
 * @method _handleDeleteButton
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to update the amount of exceptions shown in the link.
 *
 * @method _updateExceptionCounter
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to handle change event from tax select box.
 *
 * @method _handleTaxClassSelect
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to render the dialog template with the current content/data.
 *
 * @method _render
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to save the init state to restore it if dialog action gets canceled.
 *
 * @method _saveInitState
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to initialize states.
 *
 * @method _initializeState
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * Method to open the dialog.
 *
 * @method open
 * @member jQuery.ui.productUiTaxExceptions
 *
 * @since 7.63.0
 */

/**
 * @cfg {String} changeAction Server action to store choosen tax exceptions.
 */

/**
 * @cfg {Number} objectId ID of the object (product) where the exceptions are stored.
 */

/**
 * @cfg {Object} taxClassCountries Object representing all *optional* and *preselected* tax areas and their tax classes.
 */

/**
 * @cfg {Object} taxClassMappings Object representing all already choosen *optional* and *preselected* tax areas and their tax classes.
 */

/**
 * @cfg {Object} selectors Object that contains css selectors to reference needed DOM object.
 *                         Currently used: *taxClassSelect* for tax class select field
 */

/**
 * @cfg {Boolean} unity Flag to decide if Now (true) or Base (false) is present.
 */

/**
 * @cfg {Boolean} autoOpen Flag to decide if dialog should be opend on start up.
 */

/**
 * @cfg {Boolean} debug Flag to decide if an output is done in the console.
 */

/**
 * See `jQuery.ui.productUiTaxExceptions` for details.
 *
 * @param {String} changeAction Server action to store choosen tax exceptions.
 * @param {Number} objectId ID of the object (product) where the exceptions are stored.
 * @param {Object} taxClassCountries Object representing all *optional* and *preselected* tax areas and their tax classes.
 * @param {Object} taxClassMappings Object representing all already choosen *optional* and *preselected* tax areas and their tax classes.
 * @param {Object} selectors Object that contains css selectors to reference needed DOM object.
 * @param {Boolean} unity Flag to decide if Now (true) or Base (false) is present.
 * @param {Boolean} autoOpen Flag to decide if dialog should be opend on start up.
 * @param {Boolean} debug Flag to decide if an output is done in the console.
 *
 * @method productUiTaxExceptions
 * @member jQuery
 *
 * @since 7.63.0
 */

/*
 * @copyright       � Copyright 2006-2021, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/ui/tax-exceptions
 *
 * @revision        $Revision: 1.20 $
 */
/*global define, window*/
/*jslint vars:true, plusplus:true, nomen:true*/
define("de_epages/product/ui/tax-exceptions", [
    "jquery",
    "ep",
    "de_epages",
    "$dict!../dictionary",
    "$tmpl!./tax-exceptions",
    "$tmpl!./tax-exceptions-dialog",

    "ep/alert",
    "ep/ui/dialog",
    "jquery/ui/widget",
    "jquery/tmpl",
    "jquery/i18n"
], function ($, ep, de_epages, productDict, tmplTaxExceptions, tmplTaxExceptionsDialog) {
    'use strict';
     /*
     * @dictionary      ep.dict
     *
     * @translation     {Cancel}
     *                  {Country}
     *                  {Save}
     *                  {TaxClass}
     *
     * @dictionary      product.dictionary
     *
     * @translation     {TaxExceptionsButtonText}
     *                  {TaxExceptionsConfirmationDialogTitle}
     *                  {TaxExceptionsConfirmationDialogText}
     *                  {TaxExceptionsDialogChooseCountry}
     *                  {TaxExceptionsDialogTitle}
     *                  {TaxExceptionsLink}
     */

    var translations = {
            cancel: productDict.translate('Cancel'),
            country: productDict.translate('Country'),
            save: productDict.translate('Save'),
            taxClass: productDict.translate('TaxClass'),
            taxExceptionsDialogChooseCountry: productDict.translate('TaxExceptionsDialogChooseCountry'),
            taxExceptionsLink: productDict.translate('TaxExceptionsLink'),
            taxExceptionsDialogTitle: productDict.translate('TaxExceptionsDialogTitle'),
            taxExceptionsConfirmationDialogTitle: productDict.translate('TaxExceptionsConfirmationDialogTitle'),
            taxExceptionsConfirmationDialogText: productDict.translate('TaxExceptionsConfirmationDialogText'),
            taxExceptionsButtonText: productDict.translate('TaxExceptionsButtonText')
        },

        deepCopy = function(obj) {
            return JSON.parse(JSON.stringify(obj));
        };

    $.widget('ui.productUiTaxExceptions', {
    //  initialTaxClassCountries,
    //  initialTaxClassMappings,
    //  initialHide,
    //  dialog,
    //  taxClassSelect,
    //  taxClassSelectInitialIndex,

        tmplElements: {},
        hideNewExceptionRow: true,

        options: {
        //  changeAction,
        //  objectId,
        //  taxClassCountries,
            taxClassMappings: {
                preselected: [],
                optional: []
            },
            selectors: {},
            unity: true,
            autoOpen: false,
            debug: false
        },


        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _create: function () {
            var self = this,
                elem = self.element,
                o = self.options,
                exceptions;

            // if a valid taxClassCountries object exists
            if (o.taxClassCountries && o.taxClassCountries.preselected && o.taxClassCountries.optional) {
                // calculate exceptions
                exceptions = o.taxClassMappings.preselected.length + o.taxClassMappings.optional.length;
                // clone tax class countries for restauration
                self.initialTaxClassCountries = deepCopy(o.taxClassCountries);
                // set flag that decides if a new exception row is hidden
                self.hideNewExceptionRow = !!exceptions;

                // set number of exceptions
                self._updateExceptionCounter();

                // register click handler for exception link
                elem.on('click.taxExceptionLink', function() {
                    self.open();
                });

                // tax class select handling
                if (o.selectors.taxClassSelect) {
                    self.taxClassSelect = $(o.selectors.taxClassSelect);
                    self.taxClassSelect = self.taxClassSelect.length ? self.taxClassSelect : undefined;

                    if (self.taxClassSelect) {
                        // save initial select index
                        self.taxClassSelectInitialIndex = self.taxClassSelect[0].selectedIndex;
                        // set triggered flag to initial state (= alert wasn't shown yet)
                        self.taxClassSelect.data('triggered', false);

                        // register change handler
                        self.taxClassSelect.on(
                            'change.taxClassSelect',
                            {
                                widget: self
                            },
                            self._handleTaxClassSelect
                        );
                    }
                }

                o.autoOpen && self.open();
            }

        },

        /**
         * Method to create and instantiate the dialog widget.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _createDialog: function() {
            var self = this,
                elem = self.element,
                o = self.options,

                renderedDialogTmpl = tmplTaxExceptionsDialog();

            // get DOM-Elements from template
            $.extend(self.tmplElements, renderedDialogTmpl.tmplItem('elements'));

            // instantiate dialog widget
            self.dialog = $('<div/>').html(renderedDialogTmpl).uiDialog({
                autoOpen: true,
                width: 640,
                height: 560,
                modal: true,
                title: translations.taxExceptionsDialogTitle,
                classes: {
                  'ui-dialog': 'ep-tax-exceptions-dialog'
                },
                close: function(evt, ui) {
                    // close dialog via x in titlebar
                    if (evt.delegateTarget && evt.delegateTarget.classList.contains('ui-dialog-titlebar-close')) {
                        self._saveInitState();
                    }
                },
                buttons: [{
                    cssClass: 'ep-button ep-button-cancel',
                    text: translations.cancel,
                    click: function() {
                        self._saveInitState();
                        self.dialog.uiDialog("close");
                    }
                }, {
                    cssClass: 'ep-button ep-button-save',
                    text: translations.save,
                    click: function() {
                        self._send();
                    }
                }]
            });

            // START: register live events for preselected items
            // tax class select element
            self.tmplElements.dialogContent.on(
                'change.preselectedClass',
                '.ep-tax-exceptions-preselected-taxclass',
                {
                    widget: self,
                    type: 'preselected'
                },
                self._handleTaxClassSelection
            );

            // START: register live events for optional items
            // tax area select element
            self.tmplElements.dialogContent.on(
                'change.optionalArea',
                '.ep-tax-exceptions-optional-taxarea',
                {
                    widget: self,
                    type: 'optional'
                },
                self._handleTaxAreaSelection
            );
            // tax class select element
            self.tmplElements.dialogContent.on(
                'change.optionalClass',
                '.ep-tax-exceptions-optional-taxclass',
                {
                    widget: self,
                    type: 'optional'
                },
                self._handleTaxClassSelection
            );

            // START: register live events for new optional items
            // tax area select element
            self.tmplElements.dialogContent.on(
                'change.optionalArea',
                '.ep-tax-exceptions-optional-new-taxarea',
                {
                    widget: self,
                    type: 'optional'
                },
                self._handleTaxAreaSelection
            );

            // register live click event for add button
            self.tmplElements.dialogContent.on(
                'click.addButton',
                '.ep-tax-exception-add-button',
                {
                    widget: self
                },
                self._handleAddButton
            );

            // register live click event for add button
            self.tmplElements.dialogContent.on(
                'click.deleteButton',
                '.ep-tax-exceptions-delete',
                {
                    widget: self
                },
                self._handleDeleteButton
            );

            // render tax exceptions into dialog content
            self._render();
        },

        /**
         * Method to send data via an ajax request to the server.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.64.0
         */
        _send: function(evt) {
            var self = this,
                o = self.options;

            if (o.objectId && o.changeAction) {
                // execute ajax request to send data to server
                ep.ajax({
                    url: '?',
                    data: {
                        ViewAction: "JSONViewResponse",
                        ChangeAction: o.changeAction,
                        ObjectID: o.objectId,
                        TaxClassMapping: JSON.stringify(o.taxClassMappings)
                    },
                    type: "post",
                    dataType: "json"
                })
                .done(function(data, textStatus, jqXHR) {
                    if (data.success) {
                        // set number of exceptions
                        self._updateExceptionCounter();

                        self.dialog.uiDialog("close");
                    }
                })
                .fail(function(jqXHR, textStatus, errorThrown) {
                    // for debugging if an error occurs
                    o.debug && console.log("ep.ajax.fail: ", arguments);
                });
            }
        },

        /**
         * Method to change tax area and its corresponding tax classes. And trigger a re-rendering.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _handleTaxAreaSelection: function(evt) {
            var $this = $(this),
                widget = evt.data.widget,
                type = evt.data.type,
                oldCountryId = $this.data('oldCountryId'),
                newCountryId = parseInt($this.val()),
                newType = deepCopy(widget.options.taxClassCountries[type][newCountryId]),

                taxClass,
                filteredTypes;

            // get corresponding tax class from the *type* countries array
            taxClass = widget.options.taxClassCountries[type][newCountryId].taxClasses[0];
            // add tax class to new type object
            newType.taxClass = deepCopy(taxClass);
            // remove taxClasses and native properties from newType object
            delete newType.taxClasses;
            delete newType.native;

            // set flag to hide new area row if oldCountryId equals to -1
            if (oldCountryId === -1) {
                filteredTypes = deepCopy(widget.options.taxClassMappings.optional);
                // add new tax area to optional array
                widget.options.taxClassMappings.optional = filteredTypes.concat(newType);

                // hide new entry row and enable add button
                widget.hideNewExceptionRow = true;
            } else {
                // replace old tax area in optional array with new type
                widget.options.taxClassMappings.optional.forEach(function(item, index) {
                    if (oldCountryId === parseInt(item.countryId)) {
                        widget.options.taxClassMappings.optional[index] = newType;
                    }
                });
            }

            // set flag to disable the add button if last country was added
            if (Object.keys(widget.options.taxClassCountries.selects).length === 2) {
                widget.hideNewExceptionRow = false;
            }

            // re-render template with new data
            widget._render();
        },

        /**
         * Method to change tax class for chosen tax area (country).
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _handleTaxClassSelection: function(evt) {
            var $this = $(this),
                widget = evt.data.widget,
                type = evt.data.type,
                countryId = $this.data('countryId'),
                taxClassId = parseInt($this.val()),
                taxClass;

            // get corresponding tax class from the *type* countries array
            taxClass = widget.options.taxClassCountries[type][countryId].taxClasses.filter(function(item) {
                return taxClassId === item.id;
            })[0];

            // adjust tax class in taxClassMappings object
            widget.options.taxClassMappings[type].forEach(function(item) {
                // if chosen country equals to the current *type* country id
                if (countryId === parseInt(item.countryId)) {
                    item.taxClass = deepCopy(taxClass);
                }
            });
        },

        /**
         * Method to handle add button.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _handleAddButton: function(evt) {
            var widget = evt.data.widget;

            // show new optional row
            widget.hideNewExceptionRow = false;

            // re-render with current properties
            widget._render();
        },

        /**
         * Method to handle delete button.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _handleDeleteButton: function(evt) {
            var $this = $(this),
                widget = evt.data.widget,
                countryId = $this.data('countryId'),
                needCorrection = (Object.keys(widget.options.taxClassCountries.optional).length - 1)  === Object.keys(widget.options.taxClassMappings.optional).length,

                filteredTypes;

            // filter old tax area from optional array
            filteredTypes = widget.options.taxClassMappings.optional.filter(function(item) {
                return countryId !== parseInt(item.countryId);
            });

            widget.options.taxClassMappings.optional = filteredTypes;

            // don't show new optional row
            if (needCorrection) {
                widget.hideNewExceptionRow = true;
            } else if (widget.options.taxClassMappings.optional.length === 0 && widget.options.taxClassMappings.preselected.length === 0) {
                widget.hideNewExceptionRow = false;
            }

            // re-render with current properties
            widget._render();
        },

        /**
         * Method to update the amount of exceptions shown in the link.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _updateExceptionCounter: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                exceptions = o.taxClassMappings.preselected.length + o.taxClassMappings.optional.length;

            // prepare exception link with number of exceptions
            elem.html(translations.taxExceptionsLink + (exceptions ? (" (" + exceptions + ")") : ""));
        },

        /**
         * Method to handle change event from tax select box.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _handleTaxClassSelect: function(evt) {
            var widget = evt.data.widget,
                elem = widget.element;

            if (evt.target.selectedIndex !== widget.taxClassSelectInitialIndex) {
                !widget.taxClassSelect.data('triggered') && ep.alert({
                    title: translations.taxExceptionsConfirmationDialogTitle,
                    type: 'confirmation',
                    size: 'l',
                    classes: {
                      'ui-dialog': 'ep-tax-exceptions-alert'
                    },
                    content: translations.taxExceptionsConfirmationDialogText,
                    close: function(event) {
                        // do something on close
                    },
                    ok: function(event) {
                        elem.addClass('ep-tax-exceptions-link-disabled');
                        widget.taxClassSelect.data('triggered', true);
                    },
                    cancel: function(event) {
                        // reset select field to initial value
                        evt.target.selectedIndex = widget.taxClassSelectInitialIndex;
                        elem.removeClass('ep-tax-exceptions-link-disabled');
                        widget.taxClassSelect.data('triggered', false);
                    }
                });
            } else {
                // enable elem
                elem.removeClass('ep-tax-exceptions-link-disabled');
                widget.taxClassSelect.data('triggered', false);
            }
        },


        /**
         * Method to render the dialog template with the current content/data.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _render: function() {
            var self = this,
                elem = self.element,
                o = self.options,

                renderedTmpl,
                filtered;

            // filter available optional countries (tax areas) from initial tax class countries clone
            filtered = deepCopy(self.initialTaxClassCountries.optional);
            // filter already chosen countries from object
            o.taxClassMappings.optional.forEach(function(item) {
                delete filtered[item.countryId];
            });
            // set filtered list for select options
            o.taxClassCountries.selects = filtered;

            // render template with needed variables
            renderedTmpl = tmplTaxExceptions({
                taxClassCountries: o.taxClassCountries,
                taxClassMappings: o.taxClassMappings,
                unity: o.unity,
                hideNewExceptionRow: self.hideNewExceptionRow,
                formatTaxRate: function(val) {
                    return $.i18n.formatNumber(val, "p2", {region: $.i18n.settings.region});
                },
                t: translations
            });

            // get DOM-Elements from template
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));
            // render template to dialog content
            self.tmplElements.dialogContent.html(renderedTmpl);

            return renderedTmpl;
        },

        /**
         * Method to save the init state to restore it if dialog action gets canceled.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _saveInitState: function() {
            var self = this,
                o = self.options;

            o.taxClassMappings = deepCopy(self.initialTaxClassMappings);
            self.hideNewExceptionRow = self.initialHide;
        },

        /**
         * Method to initialize states.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        _initializeState: function() {
            var self = this,
                o = self.options;

            self.initialTaxClassMappings = deepCopy(o.taxClassMappings);
            self.initialHide = self.hideNewExceptionRow;
        },

        /**
         * Method to open the dialog.
         *
         * @member jQuery.ui.productUiTaxExceptions
         *
         * @since 7.63.0
         */
        open: function() {
            var self = this,
                o = self.options;

            if (self.dialog) {
               self.dialog.uiDialog('open');
            } else {
                self._createDialog();
            }

            self._initializeState();
            self._render();
        },

        destroy: function () {
            var self = this,
                elem = self.element;

            self._superApply(arguments);

            // remove events
            elem.off('click.taxExceptionLink');
            // if dialog was created and elements inside exist
            if (self.tmplElements.dialogContent) {
                self.tmplElements.dialogContent.off('change.preselectedClass');
                self.tmplElements.dialogContent.off('change.optionalArea');
                self.tmplElements.dialogContent.off('change.optionalClass');
                self.tmplElements.dialogContent.off('change.optionalArea');
                self.tmplElements.dialogContent.off('click.addButton');
                self.tmplElements.dialogContent.off('click.deleteButton');
            }
            self.taxClassSelect && self.taxClassSelect.off('change.taxClassSelect');

            // destroy widgets
            self.dialog && self.dialog.uiDialog('destroy');
        }

    });

    return de_epages;
});
