/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/progressbar-circle-test',
    'testsuite/sinon',
    'jquery',

    'ep/ui/progressbar-circle',,
    "jquery/i18n",
    '$ready!'
], function (QUnit, fixture, sinon, $) {

    // set module wide fixture property
    QUnit.fixtureVar = fixture;


    QUnit.test('ep/ui/progressbar-circle - check existence of progressbar-circle methods', 11, function () {
        $.each([
            '_create',
            '_buildUI',
            '_progress',
            'start',
            'reset',
            '_setOption',
            '_onProgressEnd',
            '_linear',
            '_sine',
            '_quad',
            'destroy'
        ], function () {
            QUnit.ok(
                $.isFunction($.ui.uiProgressbarCircle.prototype[this]),
                this + ' is a method of the progressbar-circle widget'
            );
        });
    });

    QUnit.asyncTest("ep/ui/progressbar-circle - Test basic widget functionality", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $progressbarCircle = $('.ep-progressbar-wrapper');

        // instantiate progressbar-circle widget
        $progressbarCircle.uiProgressbarCircle();

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$progressbarCircle.uiProgressbarCircle('instance'),
                true,
                'uiProgressbarCircle widget instance exists.'
            );

            QUnit.ok(
                $progressbarCircle.find('.ep-progressbar-circle').length === 1,
                'Progressbar content was appended to DOM.'
            );

            $progressbarCircle.uiProgressbarCircle('destroy');
            // wait until widget is destroyed
            window.setTimeout(function() {
                QUnit.equal(
                    $progressbarCircle.data('uiUiProgressbarCircle') === undefined,
                    true,
                    'Widget was destroyed.'
                );
                QUnit.equal(
                    $progressbarCircle.find('*').length,
                    0,
                    'Progressbar content was removed from DOM.'
                );

                fixture.destroy();
                QUnit.start();
            }, 300);
        }, 300);
    });

    QUnit.asyncTest("ep/ui/progressbar-circle - UI tests", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $progressbarCircle = $('.ep-progressbar-wrapper'),
            start = 0,
            stop = 50,
            value = 5000,
            currency = 'EUR',
            ease = 'quad',
            text = 'finished!',

            onProgressEndSpy,
            fnEaseSpy;

        // instantiate progressbar-circle widget
        $progressbarCircle.uiProgressbarCircle({
            auto: false,
            debug: false,
            rotate: 0,
            start: start,
            stop: stop,
            value: value,
            currency: currency,
            text: text,
            theme: 'light',
            radius: 45,
            fontSize: 0.8,
            strokeWidth: 7,
            ease: ease,
            speed: 3,
            colorCircleFront: '#f00',
            colorCircleBack: '#ccc'
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$progressbarCircle.uiProgressbarCircle('instance'),
                true,
                'uiProgressbarCircle widget instance exists.'
            );

            // theme *light* is used
            QUnit.ok(
                $progressbarCircle.find('.ep-progressbar-circle').hasClass('light'),
                'Progressbar uses theme "light".'
            );

            // test start value
            QUnit.equal(
                Number($('.ep-progressbar-circle-percent-num-value').text()),
                start,
                'Start value is set to ' + start +'.'
            );
            // test unit
            QUnit.equal(
                $('.ep-progressbar-circle-percent-num-unit').text(),
                currency,
                'Unit is set to "' + currency +'".'
            );

            // test text element
            QUnit.ok(
                $('.ep-progressbar-circle-text').length > 0,
                'Text element is available.'
            );
            QUnit.equal(
                $('.ep-progressbar-circle-text').css('visibility'),
                'hidden',
                'Text element is hidden.'
            );

            // register spy to investigate _onProgressEnd method
            onProgressEndSpy = sinon.spy($progressbarCircle.data('uiUiProgressbarCircle'), '_onProgressEnd');
            fnEaseSpy = sinon.spy($progressbarCircle.data('uiUiProgressbarCircle'), '_' + ease);

            // start animation
            $progressbarCircle.uiProgressbarCircle('start');

            // wait until dialog is open
            window.setTimeout(function() {
                // check if spy in *_onProgressEnd* was called
                QUnit.ok(
                    onProgressEndSpy.callCount === 1,
                    '"_onProgressEnd" method was called once.'
                );
                // check if spy in ease method was called
                QUnit.ok(
                    fnEaseSpy.callCount > 0,
                    '"_' + ease + '" ease method was called ' + fnEaseSpy.callCount + ' times.'
                );

                // test end value
                QUnit.equal(
                    $('.ep-progressbar-circle-percent-num-value').text(),
                    $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}),
                    'End value is set to ' + $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}) +'.'
                );

                // test text element
                QUnit.equal(
                    $('.ep-progressbar-circle-text').css('visibility'),
                    'visible',
                    'Text element is visible after animation ended.'
                );
                QUnit.equal(
                    $('.ep-progressbar-circle-text').text(),
                    text,
                    'Text element contains ' + text +'.'
                );

                // reset animation
                $progressbarCircle.uiProgressbarCircle('reset');
                window.setTimeout(function() {
                    // test start value
                    QUnit.equal(
                        Number($('.ep-progressbar-circle-percent-num-value').text()),
                        start,
                        'Start value was resetted to ' + start +'.'
                    );

                    // test text element
                    QUnit.equal(
                        $('.ep-progressbar-circle-text').css('visibility'),
                        'hidden',
                        'Text element is hidden again.'
                    );

                    // destroy widget
                    $progressbarCircle.uiProgressbarCircle('destroy');
                    // wait until widget is destroyed
                    window.setTimeout(function() {
                        QUnit.equal(
                            $progressbarCircle.data('uiUiProgressbarCircle') === undefined,
                            true,
                            'Widget was destroyed.'
                        );

                        fixture.destroy();
                        QUnit.start();
                    }, 300);
                }, 300);
            }, 800); // wait until animation ended
        }, 300);
    });

    QUnit.asyncTest("ep/ui/progressbar-circle - Test setting options", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $progressbarCircle = $('.ep-progressbar-wrapper'),
            start = 0,
            stop = 50,
            value = 5000,
            currency = '%',
            ease = 'linear',

            onProgressEndSpy,
            fnEaseSpy;

        // instantiate progressbar-circle widget
        $progressbarCircle.uiProgressbarCircle({
            auto: false,
            rotate: 0,
            start: start,
            stop: stop,
            value: value,
            radius: 45,
            fontSize: 0.8,
            strokeWidth: 7,
            ease: ease,
            speed: 8
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            // set options, reset widget and start
            start = 10;
            stop = 80;
            value = 3000;
            $progressbarCircle.uiProgressbarCircle('option', 'start', start);
            $progressbarCircle.uiProgressbarCircle('option', 'stop', stop);
            $progressbarCircle.uiProgressbarCircle('option', 'value', value);
            $progressbarCircle.uiProgressbarCircle('reset');
            // $progressbarCircle.uiProgressbarCircle('start');

            // test start value
            QUnit.equal(
                Number($('.ep-progressbar-circle-percent-num-value').text()),
                start,
                'After setting option "start": Start value is set to ' + start +'.'
            );
            // test unit
            QUnit.equal(
                $('.ep-progressbar-circle-percent-num-unit').text(),
                currency,
                'Unit is set to "' + currency +'".'
            );

            // test text element
            QUnit.equal(
                $('.ep-progressbar-circle-text').length,
                0,
                'No text element available.'
            );

            // register spy to investigate _onProgressEnd method
            onProgressEndSpy = sinon.spy($progressbarCircle.data('uiUiProgressbarCircle'), '_onProgressEnd');
            fnEaseSpy = sinon.spy($progressbarCircle.data('uiUiProgressbarCircle'), '_' + ease);

            // start animation
            $progressbarCircle.uiProgressbarCircle('start');

            // wait until animation ended
            window.setTimeout(function() {
                // check if spy in *_onProgressEnd* was called
                QUnit.ok(
                    onProgressEndSpy.callCount === 1,
                    '"_onProgressEnd" method was called once.'
                );
                // check if spy in ease method was called
                QUnit.ok(
                    fnEaseSpy.callCount > 0,
                    '"_' + ease + '" ease method was called ' + fnEaseSpy.callCount + ' times.'
                );

                // test end value
                QUnit.equal(
                    $('.ep-progressbar-circle-percent-num-value').text(),
                    $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}),
                    'After setting option "value": End value is set to ' + $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}) +'.'
                );

                ease = 'sine';
                $progressbarCircle.uiProgressbarCircle('option', 'ease', ease);
                fnEaseSpy = sinon.spy($progressbarCircle.data('uiUiProgressbarCircle'), '_' + ease);
                $progressbarCircle.uiProgressbarCircle('reset');
                // start animation
                $progressbarCircle.uiProgressbarCircle('start');

                // wait until animation ended
                window.setTimeout(function() {
                    // check if spy in ease method was called
                    QUnit.ok(
                        fnEaseSpy.callCount > 0,
                        'After setting option "ease": "_' + ease + '" ease method was called ' + fnEaseSpy.callCount + ' times.'
                    );

                    // reset animation
                    $progressbarCircle.uiProgressbarCircle('reset');
                    window.setTimeout(function() {
                        // test start value
                        QUnit.equal(
                            Number($('.ep-progressbar-circle-percent-num-value').text()),
                            start,
                            'Start value was resetted to ' + start +'.'
                        );

                        // destroy widget
                        $progressbarCircle.uiProgressbarCircle('destroy');
                        // wait until widget is destroyed
                        window.setTimeout(function() {
                            QUnit.equal(
                                $progressbarCircle.data('uiUiProgressbarCircle') === undefined,
                                true,
                                'Widget was destroyed.'
                            );

                            fixture.destroy();
                            QUnit.start();
                        }, 300);
                    }, 300);
                }, 400); // wait until animation ended
            }, 400); // wait until animation ended
        }, 300);
    });

    QUnit.asyncTest("ep/ui/progressbar-circle - Test edge cases", function () {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $progressbarCircle = $('.ep-progressbar-wrapper'),
            start = 0,
            stop = 0,
            value = 80,
            currency = '%',
            ease = 'linear',

            onProgressEndSpy,
            fnEaseSpy;

        // instantiate progressbar-circle widget
        $progressbarCircle.uiProgressbarCircle({
            auto: true,
            start: start,
            stop: stop,
            value: value,
            speed: 4
        });

        // wait until animation ended
        window.setTimeout(function() {

            // test end value
            QUnit.equal(
                $('.ep-progressbar-circle-percent-num-value').text(),
                $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}),
                'Rounded percentage "stop value" equals 0 AND "end value" greater than 0 => set end value is set to ' + $.i18n.formatNumber(value, "n0", {region: $.i18n.settings.region}) +'.'
            );

            // destroy widget
            $progressbarCircle.uiProgressbarCircle('destroy');
            // wait until widget is destroyed
            window.setTimeout(function() {
                QUnit.equal(
                    $progressbarCircle.data('uiUiProgressbarCircle') === undefined,
                    true,
                    'Widget was destroyed.'
                );

                fixture.destroy();
                QUnit.start();
            }, 800); // wait until animation ended
        }, 300); // wait until animation ended
    });
});